# WARNING - Generated by {fusen} from dev/flat_full.Rmd: do not edit by hand


#' Checks if an eval_report object is valid
#'
#' @param obj The object to be checked.
#' @return TRUE if the object is valid, otherwise stops the script.
#' @details This function verifies if the 'eval_report_obj' object corresponds to an eval_report object generated by the 'evaluation_report()' function. It also ensures that the object is a list containing the expected elements: 'data', 'identity', 'precision_recall', 'roc', 'counts', and 'performances'. If the object does not match these expectations, an error message is displayed.
#' @examples
#' # Using isValidEval_report
#' eval_report <- evaluation_report()
#' isValidEval_report(eval_report)
#' @export
isValidEval_report <- function(obj){
  message_err <- "'eval_report_obj' does not correspond to HTRfit eval_report_obj. 'eval_report_obj' can be generated using evaluation_report()."
  stopifnot(is.list(obj))
  
  if (all(sapply(obj, is.null))) {
    stop("All elements in 'obj' are NULL")
  }
  ## level 1 
  expected_names <- c("data", "identity", "counts", "performances")
  ## N.B : "precision_recall", "roc" names are not mandatory
  if (!all(expected_names %in% names(obj))) {
    stop(message_err)
  }
  if (!all(names(obj) %in% expected_names)){
    warning("Unexpected list element in 'eval_report_obj'")
  }
  return(TRUE)
}

#' Exports a dataframe to a specified file
#'
#' @param data The dataframe to be exported.
#' @param outfile The name of the output file.
#' @return None (the function writes to a file).
#' @examples
#' \dontrun{
#' # Using export_dataframe
#' export_dataframe(dataframe, "output.tsv")
#' }
#' @export
export_dataframe <- function(data, outfile){
  write.table(data, outfile, quote = FALSE, sep = "\t", row.names = FALSE)
}

#' Exports an eval_report object to a specified folder
#'
#' @param eval_report_obj The eval_report object to be exported.
#' @param outfolder The output folder to export the results to.
#' @param plot_format The format of the plots to export (default: "png").
#' @param ... Additional arguments to be passed to ggsave while exporting identity_modelparams. 
#' For instance use `width = 20` to adjust the width of the plot identity_modelparams.
#' @return A message indicating that the export is done.
#' @details This function exports the data contained in the eval_report object to individual files in the specified folder. The exported data includes model parameters, model dispersion, aggregated performances, performances by parameters, and evaluation plots.
#' @examples
#' \dontrun{
#' eval_report <- evaluation_report()
#' export_evaluation_report(eval_report, "output_folder")
#' }
#' @export
export_evaluation_report <- function(eval_report_obj, outfolder, plot_format = "png", ...){
      invisible(isValidEval_report(eval_report_obj))
      dir.create(file.path(outfolder), showWarnings = TRUE)
      user_workdir <- getwd()
      setwd(outfolder)
      message(paste("INFO: Exporting results in :", outfolder, sep = " " ))
      ## data export
      export_dataframe(eval_report_obj$data$modelparams, "data_modelparams.tsv")
      export_dataframe(eval_report_obj$data$modeldispersion, "data_modeldispersion.tsv")
      ## perf export
      export_dataframe(eval_report_obj$performances$aggregate, "performances_aggregated.tsv")
      export_dataframe(eval_report_obj$performances$byparams, "performances_byparams.tsv")
      export_eval_plots(eval_report_obj, plot_format, ...)
      setwd(user_workdir)
      return("Export: Done")
}

#' Exports evaluation plots from an eval_report object
#'
#' @param eval_report_obj The eval_report object containing evaluation plots.
#' @param extension The file extension/format of the exported plots.
#' @param ... Additional arguments to be passed to ggsave while exporting identity_modelparams. For instance use `width = 20` to adjust the width of the plot identity_modelparams.
#' @return None (the function writes the plots to files).
#' @details This function exports evaluation plots, including identity model parameters, identity model dispersion, precision-recall by parameters, precision-recall aggregated, ROC by parameters, ROC aggregated, and genes expression plots from the eval_report object to individual files in a 'plots' folder within the current working directory.
#' @examples
#' \dontrun{
#' export_eval_plots(eval_report, "png", width = 20)
#' }
#' @importFrom ggplot2 ggsave
#' @export
export_eval_plots <- function(eval_report_obj, extension, ...){
  
  dir.create("plots", showWarnings = FALSE)
  setwd("./plots")
  
  ggplot2::ggsave(filename = paste("identity_modelparams" , extension, sep = "."), eval_report_obj$identity$params, ...)
  ggplot2::ggsave(filename = paste("identity_modeldispersion" , extension, sep = "."), eval_report_obj$identity$dispersion )
  ggplot2::ggsave(filename = paste("genes_expression" , extension, sep = "."), eval_report_obj$counts)
  
  if (all(c("roc", "precision_recall") %in% names(eval_report_obj))){
      ggplot2::ggsave(filename = paste("precision_recall_byparams" , extension, sep = "."), eval_report_obj$precision_recall$params )
      ggplot2::ggsave(filename = paste("precision_recall_aggregated" , extension, sep = "."), eval_report_obj$precision_recall$aggregate)
      ggplot2::ggsave(filename = paste("roc_byparams" , extension, sep = "."), eval_report_obj$roc$params )
      ggplot2::ggsave(filename = paste("roc_aggregated" , extension, sep = "."), eval_report_obj$roc$aggregate )
  }

}


